/*
Copyright (C) 2009 Marco Michelino <michelinux@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <grrlib.h>
#include "main.h"
#include "cloud.h"
#include "cloud_png.h"

#define MAX_CLOUDS 3

#define INACTIVE_CLOUD 0
#define ACTIVE_CLOUD   1

#define CLOUD_WIDTH  240
#define CLOUD_HEIGHT 128

#define MAX_Y 250.0

#define MIN_BG_SPEED   0.1
#define MIN_FG_SPEED   0.3
#define SPEED_VARIANCE 0.2

struct cloud {
  float x;
  int   y;
  /* there is just one cloud image
     we have different shapes by flipping and scaling it */
  float xscale;
  float yscale;
  float vx;
  int   state;
  /* 0 = dark opaque cloud
     1 = light transparent cloud */
  int   foreground;
};

/* if you set this to 0, this module does nothing */
int use_clouds= 1;

static struct cloud clouds[MAX_CLOUDS];
static GRRLIB_texImg *cloud_image= NULL;
static u32 colors[]= {0x8888A0FF, 0xFFFFFFC0};

/* generate random number between -1 and -0.5 or between 0.5 and 1 */
static float random_scale() {
  float r;
  r= rand();
  r/= RAND_MAX;
  if (r > 0.5) {
    return r;
  } else {
    return r - 1;
  }
}

/* draw cloud number i */
static void draw_cloud(int i) {
  if (cloud_image) {
    GRRLIB_DrawImg(clouds[i].x, clouds[i].y, cloud_image, 0, clouds[i].xscale, clouds[i].yscale, colors[clouds[i].foreground]);
  }
}

/* if number of clouds < MAX_CLOUDS, create a new cloud */
int new_cloud(int x) {
  if (use_clouds) {
    int i;
    for (i= 0; i < MAX_CLOUDS; i++) {
      if (clouds[i].state == INACTIVE_CLOUD) {
        clouds[i].state= ACTIVE_CLOUD;
        clouds[i].xscale= random_scale();
        if (x == 0) {
          clouds[i].x= - abs(clouds[i].xscale * CLOUD_WIDTH);
        } else {
          clouds[i].x= x;
        }
        if (clouds[i].xscale < 0) {
          clouds[i].x-= clouds[i].xscale * CLOUD_WIDTH;
        }
        clouds[i].y= ((MAX_Y - VERTICAL_BORDER) * rand()) / (RAND_MAX + 1.0) + VERTICAL_BORDER;
        clouds[i].yscale= random_scale();
        if (clouds[i].yscale < 0) {
          clouds[i].y-= clouds[i].yscale * CLOUD_HEIGHT;
        }
        clouds[i].foreground= (2.0 * rand()) / (RAND_MAX + 1.0);
        if (clouds[i].foreground) {
          clouds[i].vx= MIN_FG_SPEED + (SPEED_VARIANCE * rand()) / (RAND_MAX + 1.0);
        } else {
          clouds[i].vx= MIN_BG_SPEED + (SPEED_VARIANCE * rand()) / (RAND_MAX + 1.0);
        }
        return 1;
      }
    }
  }
  return 0;
}

/* inizialization for this module */
void init_clouds() {
  int i;

  for (i= 0; i < MAX_CLOUDS; i++) {
    clouds[i].state= INACTIVE_CLOUD;
  }

  if (use_clouds) {
    if (!cloud_image) {
      cloud_image= GRRLIB_LoadTexture(cloud_png);
    }

    for (i= 0; i < MAX_CLOUDS; i++) {
      new_cloud(1 + ((float) (SCREEN_WIDTH - CLOUD_WIDTH) * rand()) / (RAND_MAX + 1.0));
    }
  }
}

/* draw all clouds if level == 0
   draw background clouds if level == 1 */
void draw_background_clouds() {
  int i;
  for (i= 0; i < MAX_CLOUDS; i++) {
    if (clouds[i].state == ACTIVE_CLOUD && (level == 0 || (clouds[i].foreground == 0 && level != 2))) {
      draw_cloud(i);
    }
  }
}

/* draw all clouds if level == 2
   draw foreground clouds if level == 1 */
void draw_foreground_clouds() {
  int i;
  for (i= 0; i < MAX_CLOUDS; i++) {
    if (clouds[i].state == ACTIVE_CLOUD && (level == 2 || (clouds[i].foreground == 1 && level != 0))) {
      draw_cloud(i);
    }
  }
}

/* let the wind blow */
void move_clouds() {
  int i;
  for (i= 0; i < MAX_CLOUDS; i++) {
    if (clouds[i].state == ACTIVE_CLOUD) {
      clouds[i].x+= clouds[i].vx;
      if (clouds[i].x > SCREEN_WIDTH && clouds[i].x + clouds[i].xscale * CLOUD_WIDTH > SCREEN_WIDTH) {
        clouds[i].state= INACTIVE_CLOUD;
      }
    }
  }
}

