/*
Copyright (C) 2009 Marco Michelino <michelinux@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <grrlib.h>
#include <asndlib.h>
#include "main.h"
#include "player.h"
#include "saucer.h"
#include "missile.h"
#include "explosion.h"
#include "saucer_png.h"
#include "shoot_snd.h"

#define MAX_SAUCERS 10

#define INACTIVE_SAUCER  0
#define ACTIVE_SAUCER    1
#define EXPLODING_SAUCER 2
/* ... */
#define EXPLODED_SAUCER  (EXPLODING_SAUCER + EXPLOSION_IMAGES * EXPLOSION_IMAGE_REPEAT)

#define CREATION_PAUSE     150
#define MIN_CREATION_PAUSE  50

#define MAX_Y 300.0

struct saucer {
  int x;
  int y;
  int vx;
  int vy;
  int x1;
  int y1;
  int state;
  int next_shoot;
  int color;
};

u32 saucers_colors[COLORS_NUMBER]= {0xC000FFFF, 0x00FF00FF, 0xFFC000FF, 0x6060FFFF};

static struct saucer saucers[MAX_SAUCERS];
static int creation_pause;
static int next_saucer= 0;
static GRRLIB_texImg *saucer_image= NULL;
static int initial_shoot_pause[]= {140, 100, 80};
static int shoot_pause;
static int time_to_target[]= {70, 50, 45};

/* returns number of saucers killed by shoot at cursor position */
int hit_saucers(const WPADData *wd) {
  int i;
  int killed= 0;

  if (use_sounds) {
    ASND_SetVoice(ASND_GetFirstUnusedVoice(), VOICE_MONO_8BIT, 22050, 0, (char *) shoot_snd, shoot_snd_size, MID_VOLUME, MID_VOLUME, NULL);
  }

  for (i= 0; i < MAX_SAUCERS; i++) {
    if (saucers[i].state == ACTIVE_SAUCER &&
        saucers[i].x <= wd->ir.x + 2 &&
        saucers[i].x + SAUCER_WIDTH >= wd->ir.x - 2 &&
        saucers[i].y <= wd->ir.y + 2 &&
        saucers[i].y + SAUCER_HEIGHT >= wd->ir.y - 2) {
      saucers[i].state= EXPLODING_SAUCER;
      killed++;
    }
  }

  if (killed) {
    /* game becomes a bit faster everytime you hit an enemy */
    if (creation_pause > MIN_CREATION_PAUSE) {
      creation_pause--;
    } else {
      shoot_pause--;
    }

    if (use_sounds) {
      play_explosion();
    }
  }

  return killed;
}

/* new random direction for saucer, speed depends on level and distance */
static void new_target(struct saucer *s) {
  s->x1= ((SCREEN_WIDTH - 2.0 * HORIZONTAL_BORDER - SAUCER_WIDTH) * rand()) / (RAND_MAX + 1.0) + HORIZONTAL_BORDER;
  s->y1= ((MAX_Y - VERTICAL_BORDER) * rand()) / (RAND_MAX + 1.0) + VERTICAL_BORDER;
  s->vx= (s->x1 - s->x) / time_to_target[level];
  s->vy= (s->y1 - s->y) / time_to_target[level];
}

/* if the saucer is alive move it towards his target, if is exploding let the
   explosion evolve */
static void move_saucer(struct saucer *s) {
  if (s->state == ACTIVE_SAUCER) {
    s->x+= s->vx;
    s->y+= s->vy;
    if (s->vx * (s->x1 - s->x) <= 0 || s->vy * (s->y1 - s->y) <= 0) {
      new_target(s);
    }
    if (s->next_shoot <= 0) {
      if (new_missile(s->x + SAUCER_WIDTH / 2, s->y + SAUCER_HEIGHT)) {
        s->next_shoot= shoot_pause;
      }
    } else {
      s->next_shoot--;
    }
  } else if (s->state >= EXPLODING_SAUCER &&
             s->state <= EXPLODING_SAUCER + EXPLOSION_IMAGES * EXPLOSION_IMAGE_REPEAT) {
    s->state++;
  } else if (s->state > EXPLODING_SAUCER + EXPLOSION_IMAGES * EXPLOSION_IMAGE_REPEAT) {
    s->state= INACTIVE_SAUCER;
  }
}

/* let the saucers fly */
void move_saucers() {
  int i;
  for (i= 0; i < MAX_SAUCERS; i++) {
    move_saucer(saucers+i);
  }
}

/* inizialization for this module */
void init_saucers() {
  int i;

  creation_pause= CREATION_PAUSE;
  shoot_pause= initial_shoot_pause[level];

  for (i= 0; i < MAX_SAUCERS; i++) {
    saucers[i].state= INACTIVE_SAUCER;
  }

  next_saucer= creation_pause;

  if (!saucer_image) {
    saucer_image= GRRLIB_LoadTexture(saucer_png);
  }
}

/* crate a new saucer if they are less than MAX_SAUCERS and if enough time has
   passed since previous saucer was created */
int new_saucer() {
  if (next_saucer <= 0) {
    int i;
    for (i= 0; i < MAX_SAUCERS; i++) {
      if (saucers[i].state == INACTIVE_SAUCER) {
        saucers[i].state= ACTIVE_SAUCER;
        saucers[i].next_shoot= shoot_pause;
        next_saucer= creation_pause;
        saucers[i].color= ((double) COLORS_NUMBER * rand()) / (RAND_MAX + 1.0);

        /* place new saucer at random position with random direction ad speed */
        int where_is= (100.0 * rand()) / (RAND_MAX + 1.0);
        if (where_is <= 25) {
          saucers[i].x= 0;
          saucers[i].y= MAX_Y * (25.0 - where_is) / 25;
        } else if (where_is >= 75) {
          saucers[i].x= SCREEN_WIDTH - SAUCER_WIDTH;
          saucers[i].y= MAX_Y * (100.0 - where_is) / 25;
        } else {
          saucers[i].x= (SCREEN_WIDTH - SAUCER_WIDTH) * (where_is - 25.0) / 50;
          saucers[i].y= 0;
        }
        new_target(saucers+i);

        return 1;
      }
    }
    /* too many alive saucers */
    return 0;
  } else {
    /* not enough time has passed */
    next_saucer--;
    return 0;
  }
}

/* draw active and exploding saucers according to their state */
void draw_saucers() {
  int i;
  for (i= 0; i < MAX_SAUCERS; i++) {
    if (saucers[i].state == ACTIVE_SAUCER) {
      draw_active_saucer(saucers[i].x, saucers[i].y, saucers[i].color);
    } else if (saucers[i].state >= EXPLODING_SAUCER) {
      draw_explosion(saucers[i].x, saucers[i].y, SAUCER_WIDTH, SAUCER_HEIGHT, ((saucers[i].state - EXPLODING_SAUCER) / EXPLOSION_IMAGE_REPEAT) % EXPLOSION_IMAGES);
    }
  }
}

/* draw a saucer image at (x, y) */
void draw_active_saucer(int x, int y, int color) {
  GRRLIB_DrawImg(x, y, saucer_image, 0, 1, 1, saucers_colors[color]);
}

